from dataclasses import dataclass
from mlplatform_lib.dataclass.model import ModelDto


@dataclass
class ModelAutomlDto(ModelDto):
    algorithm: str
    metric: str
    metric_result: str
    model_json: str



from mlplatform_lib.api_client import ApiClient
from mlplatform_lib.virtualization.virtualization_http_client import VirtualizationHttpClient

class VirtualizationApi:
    def __init__(self, api_client: ApiClient = None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self.virtualization_client = VirtualizationHttpClient(
            virtualization_addr=self.api_client.virtualization_addr, api_client=api_client
        )

    def download_unstructured(self, do_id: str) :
        print("Start download_unstructured in Virtualization Api")
        self.virtualization_client.download_unstructured(do_id=do_id)


from dataclasses import dataclass
from enum import Enum
import json
import requests
from typing import Optional, Dict, List, Union, Tuple
from mlplatform_lib.utils.dataclass_utils import from_dict, to_dict
from mlplatform_lib.api_client import ApiClient

class VirtualizationRequestException(Exception):
    status_code = 200

    def __init__(self, message, status_code=None):
        Exception.__init__(self)
        self.message = message
        if status_code is not None:
            self.status_code = status_code

class VirtualizationRequestType(Enum):
    CREATE = 0
    READ = 1
    UPDATE = 2
    DELETE = 3
    DOWNLOAD = 4
    READ_WITH_FORM_DATA = 5

@dataclass
class VirtualizationRequestResult:
    data: Optional[Union[Dict, List[Dict], bytes]]
    content: Optional[str]
    status_code: Optional[int]

class VirtualizationHttpClient:
    def __init__(self, virtualization_addr, api_client: ApiClient):
        self.url = virtualization_addr + "/hyperdata/web-service"
        self.api_client = api_client

    def send_request(
        self,
        service: str,
        rest: Optional[dict],
        query: Optional[dict],
        data: Optional[dict],
        request_type: VirtualizationRequestType,
    ) -> VirtualizationRequestResult:
        headers = {
            "Content-Type": "application/json",
        }

        if rest is None:
            rest = {}

        if query is None:
            query = {}

        if data is None:
            data = {}

        if service == "dataObjectTuple":
            headers["doType"] = "inferenceResult"

        if request_type == VirtualizationRequestType.DOWNLOAD:
            headers["Content-Type"] = "application/octet-stream"

        for key, val in rest.items():
            url = url + "/" + str(key) + "/" + str(val)
        if service != "" and service is not None:
            url = url + "/" + service

        if len(query) != 0:
            url += "?"
            for idx, (key, val) in enumerate(query.items()):
                if idx != 0:
                    url += "&"
                url = url + str(key) + "=" + str(val)

        if request_type == VirtualizationRequestType.READ or request_type == VirtualizationRequestType.DOWNLOAD:
            response = requests.get(url, headers=headers, data=json.dumps(data), verify=False)
        elif request_type == VirtualizationRequestType.CREATE:
            response = requests.post(url, headers=headers, data=json.dumps(data), verify=False)
        elif request_type == VirtualizationRequestType.UPDATE:
            response = requests.put(url, headers=headers, data=json.dumps(data), verify=False)
        elif request_type == VirtualizationRequestType.DELETE:
            response = requests.delete(url, headers=headers, data=json.dumps(data), verify=False)
        elif request_type == VirtualizationRequestType.READ_WITH_FORM_DATA:
            response = requests.get(url, headers=headers, data=json.dumps(data), verify=False)
        else:
            return VirtualizationRequestResult(None, None, None)

        if response.status_code == 200:
            if request_type == VirtualizationRequestType.DOWNLOAD:
                return VirtualizationRequestResult(response.content, None, response.status_code)
            else:
                return VirtualizationRequestResult(
                    response.json(), response.content.decode(), response.status_code
                )
        else:
            print(response.content.decode())
            print(
                (
                    f"cannot connect to {self.url}. "
                    f"virtualization server returns status code {response.status_code}"
                )
            )
            raise VirtualizationRequestException(response.content.decode(), response.status_code)

    def download_unstructured(self, do_id:str):
        print("Start download_unstructured in Virtualization Http Client")
        return self.send_request(
            "download/ozone/unstructured",
            {"dataobjects": do_id},
            {"fileName":do_id,"regex":"","dateStart":"","dateEnd":""},
            {},
            VirtualizationRequestType.READ,
        )


from typing import Dict
from dataclasses import dataclass


@dataclass
class InferenceAutomlDto:
    id: int
    workflow: Dict[str, str]
    experiment_type: str
    inference_path: str
    input_do_id: int
    target_do_id: int



from dataclasses import dataclass
from enum import Enum
import abc
from typing import List


class StrEnum(str, Enum):
    def __repr__(self):
        return str(self.name)

    def __str__(self):
        return str(self.name)


class PredefinedAIArgType(StrEnum):
    String = 'String'
    Integer = 'Integer'
    Float = 'Float'
    Categorical = 'Categorical'
    MultipleOf2 = 'MultipleOf2'
    Bool = 'Bool'


class PredefinedAIArg:
    @abc.abstractmethod
    def get_key(self):
        raise NotImplementedError

    @abc.abstractmethod
    def get_value(self):
        raise NotImplementedError


class ArgGroup(StrEnum):
    Arg = 'Arg'
    AlgorithmHyperparameter = 'AlgorithmHyperparameter'
    ModelHyperparameter = 'ModelHyperparameter'
    SamplingHyperparameter = 'SamplingHyperparameter'
    MetricHyperparameter = 'MetricHyperparameter'


@dataclass
class PredefinedAIStringArg(PredefinedAIArg):
    key: str
    display_name: str
    description: str
    value: str
    group: ArgGroup = ArgGroup.Arg
    type: PredefinedAIArgType = PredefinedAIArgType.String

    def get_key(self):
        return self.key

    def get_value(self):
        return self.value


@dataclass
class PredefinedAIBoolArg(PredefinedAIArg):
    key: str
    display_name: str
    description: str
    value: bool
    group: ArgGroup = ArgGroup.Arg
    type: PredefinedAIArgType = PredefinedAIArgType.Bool

    def get_key(self):
        return self.key

    def get_value(self):
        return self.value


@dataclass
class PredefinedAIIntegerArg(PredefinedAIArg):
    key: str
    display_name: str
    description: str
    min: int
    max: int
    value: int
    unit_step: int
    group: ArgGroup = ArgGroup.Arg
    type: PredefinedAIArgType = PredefinedAIArgType.Integer

    def get_key(self):
        return self.key

    def get_value(self):
        return self.value


@dataclass
class PredefinedAIFloatArg(PredefinedAIArg):
    key: str
    display_name: str
    description: str
    min: float
    max: float
    value: float
    unit_step: float
    group: ArgGroup = ArgGroup.Arg
    type: PredefinedAIArgType = PredefinedAIArgType.Float

    def get_key(self):
        return self.key

    def get_value(self):
        return self.value


@dataclass
class PredefinedAIMultipleOf2Arg(PredefinedAIArg):
    key: str
    display_name: str
    description: str
    min: int
    max: int
    value: int
    unit_step: int
    group: ArgGroup = ArgGroup.Arg
    type: PredefinedAIArgType = PredefinedAIArgType.MultipleOf2

    def get_key(self):
        return self.key

    def get_value(self):
        return self.value


@dataclass
class PredefinedAICategoricalValue:
    key: str
    display_name: str
    description: str


@dataclass
class PredefinedAICategoricalArg(PredefinedAIArg):
    key: str
    display_name: str
    description: str
    values: List[PredefinedAICategoricalValue]
    value: PredefinedAICategoricalValue
    group: ArgGroup = ArgGroup.Arg
    type: PredefinedAIArgType = PredefinedAIArgType.Categorical

    def get_key(self):
        return self.key

    def get_value(self):
        return self.value.key


from mlplatform_lib.api_client import ApiClient, RunMode
from mlplatform_lib.dataclass.experiment.type import ExperimentType
from mlplatform_lib.dataclass.model.type import ModelStatus
from mlplatform_lib.dataclass.model import ModelPredefinedaiDto, ModelInfoDto
from mlplatform_lib.dataclass import InsertTupleObject
from mlplatform_lib.dataclass.experiment import ExperimentDataObjectInfo
from mlplatform_lib.utils.dataclass_utils import from_dict
from mlplatform_lib.predefinedai.predefinedai_http_client import PredefinedAIHttpClient
from mlplatform_lib.predefinedai.predefinedai_local_checker import PredefinedAILocalChecker
import os
from typing import List, Optional
from mlplatform_lib.hyperdata import HyperdataApi
import pandas as pd
from pathlib import Path


class PredefinedAIApi:
    def __init__(self, api_client: ApiClient = None, local_mount_path="./tmp"):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self.hyperdata_api = HyperdataApi(api_client=self.api_client)
        self.experiment_id = self.api_client.experiment_id
        self.train_id = self.api_client.train_id
        self.inference_id = self.api_client.inference_id

        if api_client.run_mode == RunMode.LOCAL:
            self.local_checker = PredefinedAILocalChecker(api_client=self.api_client)
            self.local_checker.run()
            self.train_do_id = self.local_checker.train_do_id
            self.train_data_object_info_list = self.local_checker.train_data_object_info_list
            self.inference_do_id = self.local_checker.inference_do_id
            self.mount_path = local_mount_path
            Path(self.mount_path).mkdir(parents=True, exist_ok=True)
        elif api_client.run_mode == RunMode.KUBERNETES:
            self.predefinedai_client = PredefinedAIHttpClient(
                mlplatform_addr=os.environ["mlplatformAddr"], api_client=self.api_client
            )
            self.train_do_id = int(os.environ["doId"]) if "doId" in os.environ else None
            self.inference_do_id = int(os.environ["doId"]) if "doId" in os.environ else None
            self.train_data_object_info_list: List[ExperimentDataObjectInfo] = [
                from_dict(ExperimentDataObjectInfo, dict) for dict in
                os.environ["experimentDataObjectInfoList"]] if "experimentDataObjectInfoList" in os.environ else []
            self.mount_path = self.api_client.pvc_mount_path
            self.target_do_id = os.environ["targetDoId"] if "targetDoId" in os.environ else None
            if "servingMountPath" in os.environ:
                # shutil.rmtree(os.environ["mount_path"])
                self.serving_mount_path = os.environ["servingMountPath"]
                Path(os.path.dirname(self.mount_path)).mkdir(parents=True, exist_ok=True)
                os.symlink(self.serving_mount_path, self.mount_path)
            else:
                Path(self.get_experiment_path()).mkdir(parents=True, exist_ok=True)
                Path(self.get_train_path()).mkdir(parents=True, exist_ok=True)
                if self.inference_id is not None:
                    Path(self.get_inference_path()).mkdir(parents=True, exist_ok=True)

    def download_train_csv(self) -> str:
        return self.hyperdata_api.download_csv(
            do_id=int(self.train_do_id), data_rootpath=self.get_experiment_path(),
        )

    def download_train_csv_from_data_key(self, key: str) -> str:
        do_id = None
        for data_object_info in self.train_data_object_info_list:
            if data_object_info.key == key:
                do_id = data_object_info.do_id
        return self.hyperdata_api.download_csv(
            do_id=do_id, data_rootpath=self.get_experiment_path()
        )

    def download_retrain_csv(self) -> str:
        return self.download_train_csv()

    def download_inference_csv(self) -> str:
        return self.hyperdata_api.download_csv(
            do_id=int(self.inference_do_id), data_rootpath=self.get_inference_path(),
        )

    def get_train_id(self) -> int:
        return self.train_id

    def get_inference_id(self) -> int:
        return self.inference_id

    def get_experiment_path(self) -> str:
        experiment_path = os.path.join(self.mount_path, f"experiment-{self.experiment_id}")
        if "servingMountPath" not in os.environ:
            Path(experiment_path).mkdir(parents=True, exist_ok=True)
        return experiment_path

    def get_train_path(self) -> str:
        train_path = os.path.join(self.get_experiment_path(), f"train-{self.train_id}")
        if "servingMountPath" not in os.environ:
            Path(train_path).mkdir(parents=True, exist_ok=True)
        return train_path

    def get_prev_train_path(self) -> List[str]:
        return [
            os.path.join(self.get_experiment_path(), str(train_id))
            for train_id in range(1, self.get_train_id())
        ]

    def get_inference_path(self):
        inference_path = os.path.join(self.get_experiment_path(), f"inference-{self.inference_id}")
        if "servingMountPath" not in os.environ:
            Path(inference_path).mkdir(parents=True, exist_ok=True)
        return inference_path

    def get_inference_csv_path(self):
        return os.path.join(self.get_inference_path(), "inference.csv")

    def upload_model(self, model_id: int, model_path: str):
        return self.predefinedai_client.upload_model(
            experiment_id=self.experiment_id, train_id=self.train_id, model_id=model_id, model_path=model_path
        )

    def get_models(self) -> Optional[List[ModelPredefinedaiDto]]:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            return self.predefinedai_client.get_model_infos(
                experiment_id=self.api_client.experiment_id, train_id=self.api_client.train_id
            )
        else:
            print("Current mode is local, Skip get_model_infos.")
            return None

    def insert_model(self, model: ModelPredefinedaiDto) -> Optional[ModelPredefinedaiDto]:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            model.train_id = self.train_id
            model.status = ModelStatus.SUCCESS
            model.experiment_type = ExperimentType.PREDEFINEDAI
            return self.predefinedai_client.insert_model(
                experiment_id=self.experiment_id, train_id=self.train_id, model=model
            )
        else:
            print("Current mode is local, Skip insert_model_info.")
            model.id = 1
            model.train_id = self.train_id
            model.status = ModelStatus.SUCCESS
            model.experiment_type = ExperimentType.PREDEFINEDAI
            return model

    def insert_model_info(self, model_id: int, model_info: ModelInfoDto) -> ModelInfoDto:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            model_info.status = ModelStatus.SUCCESS
            return self.predefinedai_client.insert_model_info(
                experiment_id=self.experiment_id,
                train_id=self.train_id,
                model_id=model_id,
                model_info=model_info,
            )
        else:
            print("Current mode is local, Skip insert_visualizations.")

    def _get_inference_result_table_name(self) -> str:
        pdai_experiment_dto = self.predefinedai_client.get_experiment(experiment_id=self.experiment_id)

        return f"PREDEFINEDAI_{pdai_experiment_dto.name}_{str(pdai_experiment_dto.id)}"

    def upload_inference_csv(self, inference_csv_path: str):
        if self.api_client.run_mode == RunMode.KUBERNETES:
            # 1. check is target do exists
            inference_data = pd.read_csv(inference_csv_path)

            table_name = self._get_inference_result_table_name()
            self.target_do_id = self.hyperdata_api.create_inference_result(
                table_name=table_name, object_name=table_name, columns=inference_data.columns.tolist()
            )

            insert_tuple_object = InsertTupleObject(
                isTruncated="True",
                targetColNames=inference_data.columns.tolist(),
                tableData=inference_data.values.tolist(),
            )

            self.hyperdata_api.hyperdata_client.insert_dataobject_tuple(
                dataobject_id=self.target_do_id, insert_tuple_objects=insert_tuple_object,
            )

            pdai_inference_dto = self.predefinedai_client.get_inference(
                experiment_id=self.experiment_id, inference_id=self.inference_id
            )
            pdai_inference_dto.target_do_id = self.target_do_id
            self.predefinedai_client.update_inference(
                experiment_id=self.experiment_id, inference_predefinedai_dto=pdai_inference_dto
            )

            os.remove(inference_csv_path)
        else:
            print("Current mode is local, Skip upload_inference_csv.")



from mlplatform_lib.api_client import ApiClient, RunMode
from mlplatform_lib.mllab.mllab_http_client import MllabHttpClient
from mlplatform_lib.hyperdata.hyperdata_http_client import HyperdataHttpClient
from mlplatform_lib.hyperdata.hyperdata_api import HyperdataApi
from mlplatform_lib.virtualization.virtualization_api import VirtualizationApi
from mlplatform_lib.dataclass.model.model_dto import ModelDto
from mlplatform_lib.dataclass import (
    InsertTupleObject,
    TableColumnInfo,
)
import os
import pandas as pd
import sys
from typing import List, Optional, Union


class MllabImageType:
    TENSORFLOW_V1 = "tf_v1.15.2"
    TENSORFLOW_V2 = "tf_v2.1.0"
    TORCH = "torch_v1.6.0"


if "image" in os.environ:
    if (
        os.environ["image"] == MllabImageType.TENSORFLOW_V1
        or os.environ["image"] == MllabImageType.TENSORFLOW_V2
    ):
        import tensorflow as tf
    elif os.environ["image"] == MllabImageType.TORCH:
        import torch


class MllabApi:
    def __init__(self, api_client: ApiClient = None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        if api_client.run_mode == RunMode.KUBERNETES:
            self.mllab_mlplatform_client = MllabHttpClient(
                mlplatform_addr=os.environ["mlplatformAddr"], api_client=api_client
            )
            self.mllab_hyperdata_client = HyperdataHttpClient(
                hd_addr=self.api_client.hyperdata_addr, api_client=self.api_client
            )
            self.mllab_hyperdata_api = HyperdataApi(api_client=self.api_client)
            self.mllab_virtualization_api = VirtualizationApi(api_client=self.api_client)
        self.experiment_id = self.api_client.experiment_id
        self.image = os.environ["image"]

    def get_model_list(self) -> Optional[List[ModelDto]]:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            return self.mllab_mlplatform_client.get_model_list(experiment_id=self.experiment_id)
        else:
            print("Current mode is local, Skip get_list_model.")
            return None

    def get_model(self):
        if self.api_client.run_mode == RunMode.KUBERNETES:
            
            model_path = self.api_client.pvc_mount_path + "/model/model-" + self.api_client.train_id + "/1"

            if self.image == MllabImageType.TENSORFLOW_V1:
                return tf.compat.v2.saved_model.load(model_path + "/")
            elif self.image == MllabImageType.TENSORFLOW_V2:
                return tf.saved_model.load(model_path + "/")
            elif self.image == MllabImageType.TORCH:
                sys.path.append(model_path + "/")
                return torch.jit.load(model_path + "/" + "model.pt")
        else:
            print("Current mode is local, Skip get_list_do.")
            return None

    def save_model(self, model, signatures=None) -> bool:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            export_dir = self.api_client.pvc_mount_path + "/model/model-" + self.api_client.train_id
            model_dir = export_dir + "/1"
            inference_dir = export_dir + "/inference"
            if not os.path.isdir(model_dir):
                os.makedirs(model_dir)
            if not os.path.isdir(inference_dir):
                os.makedirs(inference_dir)

            if self.image == MllabImageType.TENSORFLOW_V1:
                tf.saved_model.save(model, model_dir, signatures)
            elif self.image == MllabImageType.TENSORFLOW_V2:
                tf.saved_model.save(model, model_dir, signatures)
            elif self.image == MllabImageType.TORCH:
                jit_saved = torch.jit.script(model)
                jit_saved.save(model_dir + "/model.pt")

            model_dto = self.mllab_mlplatform_client.get_model_by_id(
                experiment_id=self.experiment_id, train_id=self.api_client.train_id
            )
            model_dto.model_path = model_dir
            self.mllab_mlplatform_client.update_model(
                experiment_id=self.experiment_id, train_id=self.api_client.train_id, dto=model_dto
            )
        else:
            print("Current mode is local, Skip get_list_do.")
            return None

    def get_do_list(self) -> dict:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            return self.mllab_hyperdata_client.get_do_simple_list()
        else:
            print("Current mode is local, Skip get_list_do.")
            return None

    def get_do(self, do_id: str, char_encoding="euc-kr") -> pd.DataFrame:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            file_name = do_id + ".csv"
            dir_name = self.api_client.pvc_mount_path + "/data"
            full_path = dir_name + "/" + file_name
            if os.path.exists(full_path):
                result = pd.read_csv(full_path, encoding=char_encoding)
                return result

            do_path = self.mllab_hyperdata_api.download_csv(do_id=do_id, data_rootpath=dir_name)

            if os.path.exists(do_path):
                result = pd.read_csv(do_path, encoding=char_encoding)
            else:
                print("Fail")
                result = None
            return result
        else:
            print("Current mode is local, Skip get_do.")
            return None

    def get_do_test(self, do_id: str) :
        print("Start get_do_test")
        self.mllab_virtualization_api.download_unstructured(do_id=do_id)

    def get_do_meta(self, do_id: str) -> List[TableColumnInfo]:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            tableDescInfo = self.mllab_hyperdata_client.get_do_detail_info(do_id=do_id)
            return tableDescInfo.col_info_list
        else:
            print("Current mode is local, Skip get_do_meta.")
            return None

    def _get_inference_result_table_name(self, inference_id: int) -> str:
        mllab_experiment_dto = self.mllab_mlplatform_client.get_experiment(experiment_id=self.experiment_id)
        return f"MLLAB_{mllab_experiment_dto.name}_{str(mllab_experiment_dto.id)}"

    def save_inference_result(self, input_data: pd.DataFrame, do_id: Optional[str]= None, is_truncated: bool = True) -> bool:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            if self.api_client.inference_id is not None:
                file_dir = (
                    self.api_client.pvc_mount_path + "/model/model-" + self.api_client.train_id + "/inference/inference-" + self.api_client.inference_id
                )
                file_path = file_dir + "/inference-result.csv"
                if not os.path.isdir(file_dir):
                    os.mkdir(file_dir)
                input_data.to_csv(file_path, index=False, header=True)
            else:
                print("inference id is None")
                return False
        
            inference_dto = self.mllab_mlplatform_client.get_inference_by_id(
                experiment_id=self.experiment_id, train_id=self.api_client.train_id, model_id=self.api_client.model_id,inference_id=self.api_client.inference_id
            )
            inference_dto.inference_path = file_path
            self.mllab_mlplatform_client.update_inference( experiment_id=self.experiment_id, train_id=self.api_client.train_id, model_id=self.api_client.model_id, dto=inference_dto)

            if(do_id is None):
                 #create do in default source to save inference result
                table_name = self._get_inference_result_table_name(inference_dto.id)
                do_id = self.mllab_hyperdata_api.create_inference_result(
                    table_name=table_name, object_name=table_name, columns=input_data.columns.tolist()
                )

            insert_tuple_object = InsertTupleObject(
                isTruncated=is_truncated,
                targetColNames=input_data.columns.tolist(),
                tableData=input_data.values.tolist(),
            )
            res = self.mllab_hyperdata_client.insert_dataobject_tuple(
                dataobject_id=do_id, insert_tuple_objects=insert_tuple_object
            )
            if res.status_code == 200:
                return True
            else:
                return False
        else:
            print("Current mode is local, Skip save_inference_result.")

    

        



from dataclasses import dataclass
from typing import List, Union
from .predefinedai_arg import PredefinedAIBoolArg, PredefinedAIIntegerArg, PredefinedAIFloatArg, PredefinedAIMultipleOf2Arg, PredefinedAICategoricalArg

@dataclass
class ModelConfig:
    key: str
    display_name: str
    description: str
    hyperparameters: List[
        Union[PredefinedAIBoolArg, PredefinedAIIntegerArg, PredefinedAIFloatArg, PredefinedAIMultipleOf2Arg, PredefinedAICategoricalArg]]

    def __post_init__(self):
        self.hyperparameter_dict = {}
        for hp in self.__getattribute__('hyperparameters'):
            self.hyperparameter_dict[hp.get_key()] = hp.get_value()


@dataclass
class PredefinedAIModelConfig:
    display_name: str
    key: str
    description: str
    model_configs: List[ModelConfig]

    def __post_init__(self):
        self.hyperparameter_dict = {}
        for model_config in self.model_configs:
            self.hyperparameter_dict[model_config.key] = {}
            for hp in model_config.hyperparameters:
                self.hyperparameter_dict[model_config.key][hp.get_key()] = hp.get_value()

    def _get_model_hyperparameters_dict(self, model_name) -> dict:
        if model_name in self.hyperparameter_dict.keys():
            return self.hyperparameter_dict[model_name]
        raise ValueError(f'model "{model_name}" dose not exist in model_configs')



from dataclasses import dataclass
from typing import List, Union
from .predefinedai_arg import PredefinedAIStringArg, PredefinedAIBoolArg, PredefinedAIIntegerArg, PredefinedAIFloatArg, PredefinedAIMultipleOf2Arg, PredefinedAICategoricalArg


@dataclass
class StageConfig:
    args: List[
        Union[
            PredefinedAIStringArg, PredefinedAIBoolArg, PredefinedAIIntegerArg, PredefinedAIFloatArg, PredefinedAIMultipleOf2Arg, PredefinedAICategoricalArg]]
    envs: List[
        Union[
            PredefinedAIStringArg, PredefinedAIBoolArg, PredefinedAIIntegerArg, PredefinedAIFloatArg, PredefinedAIMultipleOf2Arg, PredefinedAICategoricalArg]]


@dataclass
class PredefinedAIStageConfig:
    train: StageConfig
    retrain: StageConfig
    inference: StageConfig
    serving: StageConfig

    def __post_init__(self):
        self.args_dict = {}
        for stage in ['train', 'retrain', 'inference', 'serving']:
            self.args_dict[stage] = {}
            for arg in self.__getattribute__(stage).__getattribute__('args'):
                self.args_dict[stage][arg.get_key()] = arg.get_value()

    def _get_args_dict(self, stage) -> dict:
        if stage in ['train', 'retrain', 'inference', 'serving']:
            return self.args_dict[stage]
        raise ValueError(f'stage "{stage}" dose not exist')
